/*
** Module   : file_system_helper_win.c
** Abstract : implementation for detection of simbolic links, non-FIFO pipes and special devices.
**            This is used by filelys.c
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130329 Created initial version to implement Windows specifics for file system
**                  operations.
*/
#include <windows.h>
#include "filesys.h"

/**
* The helper function to get symlink status in Windows. This should work beginning from Vista.
*
* @param  env 
*         pointer to a structure that provides access to JNI environment.
* @param  fname 
*         The file name to be checked passed from FileChecker.checkFileType().
* @param  file_path 
*         The pointer to file path string.
* @param  status_initial 
*         The current status value for the given file.
*
* @return An integer modified status code of the file.
*/
int getSymLinkStatus(JNIEnv *env, const char *file_path, int status_initial)
{
   // Start with currently available information
   int status_return = status_initial;

   // Structure go get file status info   
   struct stat64 buf;

   //get symbolic link status
   //we can't use fstat for detecting symbolic links
   int file_stat = _stat64(file_path, &buf); 
   
   //if file status is -1 return -1
   if (file_stat == -1) return -1;
   
   // check for special devices
   if (S_ISCHR(buf.st_mode) || S_ISBLK(buf.st_mode))
   { 
      // status code for devices is standalone
      status_return |= DEVICE;
   }
   // check for symbolic links
   // NOTE: A symbolic link that points to a directory or file will not be
   // detected as file or directory by the logic present here so this status 
   //cannot be made inclusive with file or directory
   if ((GetFileAttributes(file_path) & FILE_ATTRIBUTE_REPARSE_POINT) ==
       FILE_ATTRIBUTE_REPARSE_POINT)
   {
      //status code for symbolic link is additive
      status_return |= SYMLINK;
   }
   
   return status_return;
}

/**
* The helper function to open file or pipe in Windows.
*
* @param  filename 
*         The name of the file to open.
* @param  flags 
*         The open mode flags common across all operating systems.
*
* @return An integer file fescriptor or -1 in case of error.
*/
int open_native(const char * filename, int flags)
{
   return open(filename, flags | P_NOWAIT);
}
